*  Program...........: TABS.PRG
*  Author............: Steven M. Black
*  Version...........: 2.0a October 17 1994
*    Steven Black Consulting / UP!
*    2 Bay Street
*    Kingston, ON
*    K7K 6T7 Canada
*    (613) 542-3293
*    (613) 531-9481 (Fax)
*    Compuserve 76200,2110
*
*  With thanks to....:
*    Bill Anderson             72712,3417
*    Ken Levy [FLASH]          76350,2610
*    Andrew Ross MacNeill      76100,2725
*    Randy Pearson [CYCLA]     73020,642
*    Dan Pollak                71172,2373
*    Bob Showalter             72220,466
*    Rick Whitt [ITS]          70672,605
*    ... and many others.
*
*} Project...........: GENSCRNX drivers
*  Created...........: 05/26/94
*  Copyright.........: (c) SBC / UP!
*    This software is provided "as is" without
*    any warranty expressed or implied. In no event shall
*    Steven Black, contributors, or distributors be liable
*    for any damages.
*
*    This software requires GENSCRNX 2.0
*    and 3D 2.0, two public domain programs created by
*    Ken Levy at the JPL in Pasadena, California.
*
*    This software has been made PUBLIC DOMAIN by the author.
*
*) Description.......: Takes a screen set and creates a tab
*)                     form for a "file card" look.
*
*] Dependencies......: FP Windows 2.6 or greater
*]                     GENSCRNX   2.0 or greater
*]                     3D Driver  2.0 or greater
*]
*  Calling Samples...: *:SCXDRV5 TABS in screen 1 of a screen set
*  Parameter List....: -
*  Returns...........: -
*
*  Invoking TABS.....:
*    TABS can be invoked from CONFIG.FP and will ignore
*    screen sets not containing "*:TAB" in screen1.setupcode.
*
*    Call INTL and 3D after TABS
*     Example:
*        *:SCXDRV5 TABS
*        *:SCXDRV5 INTL
*        *:SCXDRV5 3D
*        *:ALL 3D
*
*  TABS Directives...:
*
*    First Screen Only  *:DIRECTIVE      [default]
*    ---------------------------------------------
*    *:DIRECTIVE     [default]
*
*    *:TABTITLE               - Window title
*    *:TABMARGIN     [5]      - Container window to inner-form
*    *:TABSCXMARGIN  [3]      - inner-form to SCXes
*    *:TABHEADER     [3]      - Container title to top of tabs
*    *:TABSCXHEADER  [1]      - Bottom of tabs to SCXes.
*    *:TABFOOTER     [2]      - Form to bottom of container window
*    *:TABSCXFOOTER  [1]      - Bottom of tallest SCX to bottom of form
*    *:TABWIDTH      [20]     - Tab widths (default for whole screenset)
*    *:TABSPACING    [0.25]   - Tab spacing
*    *:TABPEN        [3]      - Form boundary pen size
*    *:TABSTACK      [OFF]    - Cycle to next/prev form?
*    *:TABWINDOWNAME          - Name for the container window
*    *:TABMINIMIZE   [OFF]    - allow container minimization
*    *:TABNOCLOSE             - disallow container close
*    *:TABNOFLOAT             - disallow container window to float
*    *:TABZOOM                - allow container zoom
*    *:TABDOUBLE              - container border DOUBLE minimization
*    *:TABHEIGHT     [1.5]
*    *:TABTEXTPOS    [CENTER] - Tab text alignment. Other Option is LEFT.
*    *:TABWCLAUSE              - window clauses for container window
*
*
*
*    All screens
*    -----------
*    *:TABLABEL             - Label for the tab (Defaults to the SCX title)
*    *:TABICON              - Icon instead of/combined with text
*    *:TABWIDTH             - Override the default width
*    *:TABSKIP              - Don't include screen in tab set
*
*
*    Any object comment, first screen
*    -------------------------
*    *:TABREPLICATE           - replicate object into subscequent screens
*
*    Any screen setup, screen > 1
*    -------------------------
*    *:TABREPLICATE OFF       - Don't replicate objects into this screens
*

#DEFINE ccNULL           CHR(0)
*-- =============================================
*-- SCX objtypes
*-- =============================================
#DEFINE cnHeader             1
#DEFINE cnText               5
#DEFINE cnLine               6
#DEFINE cnBox                7
#DEFINE cnList              11
#DEFINE cnPushButton        12
#DEFINE cnRadioButton       13
#DEFINE cnCheckBox          14
#DEFINE cnSayGet            15
#DEFINE cnPopUp             16
#DEFINE cnPicture           17
#DEFINE cnSpinner           22
#DEFINE cnInvButton         20


*-- Bailout: TABS can now be called from Fonfig.FPW and will ignore screens without
*-- "*:TAB" in the setup of the first screen in the set
IF m.scxcount = 1 AND ! "*:TAB" $ setupcode
   GO BOTTOM
   RETURN
ENDIF
IF m.scxCount <> 1 AND TYPE( "m.gnWINHeight") = "U"
   GO BOTTOM
   RETURN
ENDIF

*-- Bailout: Platform
IF INLIST( m.platform_ , "DOS", "UNIX")
   =warning('This TABS driver is for GUI versions of FoxPro')
   GO BOTTOM
   RETURN .F.
ENDIF


*-- Bailout: Hook number
IF m.drv_no # 5
  GOTO BOTTOM
  =warning('The TABS driver requires *:SCXDRV5')
  RETURN .F.
ENDIF

*-- Bailout: 3D presence and sequencing
IF DrvAudit("3D") < 1
      =WARNING ("The TABS Driver must be invoked before 3D")
      GO BOTTOM
      RETURN .F.
ENDIF

*-- Bailout if m.loop_flag is true to prevent code
*-- from being executed again because of library objects
*--  With thanks to Dan Pollak
IF m.loop_flag
  GO BOTTOM
  RETURN .F.
ENDIF

*PRIVATE ALL LIKE j*, cn*, cc*, c_*, m_*
PRIVATE ;
   ccTABStack, ;
   ccTABTextPos, ;
   ccTABWClause, ;
   ccTABWindowname ,;
   cnTABFooter, ;
   cnTabHeader, ;
   cnTABHeight, ;
   cnTABMargin, ;
   cnTABPen, ;
   cnTABSCXFooter, ;
   cnTABSCXHeader, ;
   cnTABSCXMargin, ;
   cnTABSpacing, ;
   cnTABWidth, ;
   c_TabDouble, ;
   c_TabFooter, ;
   c_TabHeader, ;
   c_TabHeight, ;
   c_TabLabel, ;
   c_TabMargin, ;
   c_TabMinimize, ;
   c_TabNoClose, ;
   c_TabNoFloat, ;
   c_TabPEN, ;
   c_TabSCXFooter, ;
   c_TabSCXHeader, ;
   c_TabSCXMargin, ;
   c_TabSkip, ;
   c_TabSpacing, ;
   c_TabSTACK, ;
   c_TabTextPos, ;
   c_TabTitle, ;
   c_TabWidth, ;
   c_TabWClause, ;
   c_TabWindowName, ;
   c_TabZoom, ;
   jcCntnrProps, ;
   jcFileName, ;
   jcScnNo, ;
   jcSetup2, ;
   jcSetup, ;
   jcSetup1, ;
   jcTitle, ;
   jcWClause, ;
   jcWindowName, ;
   jnAtPos1, ;
   jnAtPos2, ;
   jnBoxHeight, ;
   jnBoxVPos, ;
   jnBoxWidth, ;
   jnCardWidth, ;
   jnHFudge, ;
   jnIBVShrink, ;
   jni, ;
   jnNumTabs, ;
   jnOldArea, ;
   jnOldRec, ;
   jnVFudge, ;
   jnWidth, ;
   m_TabDouble, ;
   m_TabFooter, ;
   m_TabHeader, ;
   m_TabHeight, ;
   m_TabLabel, ;
   m_TabMargin, ;
   m_TabMinimize, ;
   m_TabNoClose, ;
   m_TabNoFloat, ;
   m_TabPEN, ;
   m_TabSCXFooter, ;
   m_TabSCXHeader, ;
   m_TabSCXMargin, ;
   m_TabSkip, ;
   m_TabSpacing, ;
   m_TabSTACK, ;
   m_TabTextPos, ;
   m_TabTitle, ;
   m_TabWidth, ;
   m_TabWClause, ;
   m_TabWindowName, ;
   m_TabZoom

*-- =============================================
*-- These default measurements are in CHARACTERS
*-- =============================================
m.cnTABMargin       =  5
m.cnTABFooter       =  2
m.cnTabHeader       =  3
m.cnTABSCXMargin    =  3
m.cnTABSCXFooter    =  1
m.cnTABSCXHeader    =  1
m.cnTABWidth        = 20
m.cnTABSpacing      =  0.25
m.cnTABHeight       =  1.5

*-- PENSIZE for the TABS outline
m.cnTABPen          =  3

*-- =============================================
*-- ON/OFF calls
*-- =============================================
m.ccTABStack     = "OFF"
m.ccTABTextPos   = "CENTER"

m.ccTABWindowName= LOWER( SYS( 2015))
m.ccTABWClause   = ''

*-- =============================================
*-- Directives
*-- =============================================
m.c_TabTitle     = "*:TABTITLE"
m.m_TabTitle     = "*-:TABTITLE"
m.c_TabLabel     = "*:TABLABEL"
m.m_TabLabel     = "*-:TABLABEL"
m.c_TabMargin    = "*:TABMARGIN"
m.m_TabMargin    = "*-:TABMARGIN"
m.c_TabHeader    = "*:TABHEADER"
m.m_TabHeader    = "*-:TABHEADER"
m.c_TabFooter    = "*:TABFOOTER"
m.m_TabFooter    = "*-:TABFOOTER"
m.c_TabSCXMargin = "*:TABSCXMARGIN"
m.m_TabSCXMargin = "*-:TABSCXMARGIN"
m.c_TabSCXHeader = "*:TABSCXHEADER"
m.m_TabSCXHeader = "*-:TABSCXHEADER"
m.c_TabSCXFooter = "*:TABSCXFOOTER"
m.m_TabSCXFooter = "*-:TABSCXFOOTER"
m.c_TabHeight    = "*:TABHEIGHT"
m.m_TabHeight    = "*-:TABHEIGHT"
m.c_TabWidth     = "*:TABWIDTH"
m.m_TabWidth     = "*-:TABWIDTH"
m.c_TabSpacing   = "*:TABSPACING"
m.m_TabSpacing   = "*-:TABSPACING"
m.c_TabPEN       = "*:TABPEN"
m.m_TabPEN       = "*-:TABPEN"
m.c_TabSTACK     = "*:TABSTACK"
m.m_TabSTACK     = "*-:TABSTACK"
m.c_TabNoFloat   = "*:TABNOFLOAT"
m.m_TabNoFloat   = "*-:TABNOFLOAT"
m.c_TabNoClose   = "*:TABNOCLOSE"
m.m_TabNoClose   = "*-:TABNOCLOSE"
m.c_TabZoom      = "*:TABZOOM"
m.m_TabZoom      = "*-:TABZOOM"
m.c_TabMinimize  = "*:TABMINIMIZE"
m.m_TabMinimize  = "*-:TABMINIMIZE"
m.c_TabDouble    = "*:TABDOUBLE"
m.m_TabDouble    = "*-:TABDOUBLE"
m.c_TabTextPos   = "*:TABTEXTPOS"
m.m_TabTextPos   = "*-:TABTEXTPOS"
m.c_tabIcon      = "*:TABICON"
m.m_tabIcon      = "*-:TABICON"
m.c_TabWindowName= "*:TABWINDOWNAME"
m.m_TabWindowName= "*-:TABWINDOWNAME"
m.c_TabSkip      = "*:TABSKIP"
m.m_TabSkip      = "*-:TABSKIP"
m.c_TabWClause   = "*:TABWCLAUSE"
m.m_TabWClause   = "*-:TABWCLAUSE"

IF objtype = 1

  *-- If first screen in the set
  IF m.scxcount = 1

     *-- Initialize screen-size memvars
     RELEASE m.gnWINHeight,   m.gnWINWidth,    m.gnTBMargin,    ;
             m.gnTBHeader,    m.gnTBFooter,    m.gnTBSCXMargin, ;
             m.gnTBSCXHeader, m.gnTBSCXFooter, m.gnTBWidth,      ;
             m.gnTBSpacing,   m.gnTBHeight,    m.gnTBPen,       ;
             m.gcTBStack,     m.gcTBWindowName, m.gcTBTextPos

     PUBLIC  m.gnWINHeight,   m.gnWINWidth,    m.gnTBMargin,    ;
             m.gnTBHeader,    m.gnTBFooter,    m.gnTBSCXMargin, ;
             m.gnTBSCXHeader, m.gnTBSCXFooter, m.gnTBWidth,      ;
             m.gnTBSpacing,   m.gnTBHeight,    m.gnTBPen,       ;
             m.gcTBStack,     m.gcTBWindowName, m.gcTBTextPos

     RELEASE gaTABLabel, gaFont, gaTabIcon, gaTABWidths, gaSkipped
     PUBLIC ARRAY gaTABLabel[ m.screens], ;
                  gaFont[ m.screens, 6], ;
                  gaTabIcon[ m.screens], ;
                  gaTABWidths[ m.screens], ;
                  gaSkipped[ m.screens]

     STORE '' TO gaTABlabel, ;
                 gaFont, ;
                 gaTabIcon

     *-- Reckon the container window name
     m.gcTBWindowName= resolve( "WindowName", .T.)

     *-- Reckon the window clauses for container window
     m.jcWClause = resolve( "WClause", .T.)

     *-- Reckon the title
     m.jcTitle = labeltxt( c_TABTitle, '')

     REPLACE setupcode WITH strtranc( setupcode, m.c_TabTitle, m.m_TabTitle)

     *-- Reckon the metrics
     m.gnTBMargin   = resolve( "Margin")
     m.gnTBFooter   = resolve( "Footer")
     m.gnTBHeader   = resolve( "Header")
     m.gnTBSCXMargin= resolve( "SCXMargin")
     m.gnTBSCXFooter= resolve( "SCXFooter")
     m.gnTBSCXHeader= resolve( "SCXHeader")
     m.gnTBHeight   = resolve( "Height")
     m.gnTBWidth    = resolve( "Width")
     m.gnTBSpacing  = resolve( "Spacing")
     m.gnTBPen      = resolve( "Pen")
     m.gcTBStack    = resolve( "Stack", .T.)
     m.gcTBTextPos  = resolve( "TextPos", .T.)

     m.gaTABWidths  = gnTBWidth

     m.gnWINHeight   = 0
     m.gnWINWidth    = (m.screens * gnTBWidth) + ((m.screens-1) * (gnTBSpacing))

     *-- =============================================
     *--    1. Assemble Information
     *-- =============================================
     m.jnOldArea = SELECT()

     SELECT PjxData

     *-- Modification suggestion by Dan Pollak
     *-- If modal, add container window as anassociated window
     IF PjxData.Modal
        REPLACE assocwinds WITH  gcTBWindowName + ;
                                 IIF( EMPTY( assocwinds), ;
                                      '', ;
                                      ',' + assocwinds)
     ENDIF

     *-- Remember where we are
     m.jnOldRec = RECNO()
     m.jni      = 0
     SCAN FOR type == "s"
        m.jnI = m.jnI + 1
        m.jcFileName = ALLTRIM( NAME)


        SELECT 0
        USE ( m.jcFileName) AGAIN
        LOCATE FOR PLATFORM= m.platform_

        * Check for and bypass screens containing *:TABSKIP
        * With thanks to Dan Pollak
        IF WORDSEARCH( m.c_TabSkip, .T.) <> ccNull
          IF m.jnI = 1
            WAIT WINDOW 'TABS cannot *:TABSKIP the first screen.  Directive ignored.' TIME 3
          ELSE
            gaSkipped[ m.jnI] = .T.
            USE
            LOOP
          ENDIF
        ENDIF

        *-- We take control of the arrangement
        REPLACE PJXData.ARRANGED WITH ALLTRIM( m.platform_) + ;
                                      " FF 0.000 0.000"

        *-- =============================================
        *-- Screen font normalization
        *-- =============================================
        gaFont[ m.jnI, 1] = m.jnI
        gaFont[ m.jnI, 2] = 1               && Height Factor
        gaFont[ m.jnI, 3] = 1               && Width  Factor
        gaFont[ m.jnI, 4] = TRIM( fontface)
        gaFont[ m.jnI, 5] = fontsize
        gaFont[ m.jnI, 6] = fontstyle

        IF m.jnI > 1
           gaFont[ m.jnI, 2] = (FONTMETRIC( 1, ;
                                            TRIM( gaFont[ jnI, 4]), ;
                                            gaFont[ jnI, 5], ;
                                            cfontstyle(gaFont[ jnI, 6])) + ;
                                FONTMETRIC( 4, ;
                                            TRIM( gaFont[ jnI, 4]), ;
                                            gaFont[ jnI, 5], ;
                                            cfontstyle(gaFont[ jnI, 6])) + ;
                                FONTMETRIC( 5, ;
                                            TRIM( gaFont[ jnI, 4]), ;
                                            gaFont[ jnI, 5], ;
                                            cfontstyle(gaFont[ jnI, 6]))) ;
                             / ;
                               (FONTMETRIC( 1, ;
                                            TRIM( gaFont[ 1, 4]), ;
                                            gaFont[ 1, 5], ;
                                            cfontstyle(gaFont[ 1, 6])) + ;
                                FONTMETRIC( 4, ;
                                            TRIM( gaFont[ 1, 4]), ;
                                            gaFont[ 1, 5], ;
                                            cfontstyle(gaFont[ 1, 6])) + ;
                                FONTMETRIC( 5, ;
                                            TRIM( gaFont[ 1, 4]), ;
                                            gaFont[ 1, 5], ;
                                            cfontstyle(gaFont[ 1, 6])))

           gaFont[ m.jnI, 3] =  FONTMETRIC( 6, ;
                                            TRIM( gaFont[ jnI, 4]), ;
                                            gaFont[ jnI, 5], ;
                                            cfontstyle(gaFont[ jnI, 6])) / ;
                                FONTMETRIC( 6, ;
                                            TRIM( gaFont[ 1,   4]), ;
                                            gaFont[ 1,   5], ;
                                            cfontstyle(gaFont[ 1,   6]))

        ENDIF

        m.gnWINHeight = MAX( m.gnWINHeight, Height * gaFont[ m.jnI, 2])
        m.gnWINWidth  = MAX( m.gnWINWidth,   Width * gaFont[ m.jnI, 3])

        *-- Assign the TAB labels
        gaTABLabel[ m.jnI] = labeltxt( m.c_TABLabel, ;
                                       "Screen " + ALLTRIM( STR( m.jnI)))

        gaTABIcon[ m.jnI]  = icontxt( m.c_TABIcon, '')

        *-- Tab widths
        *-- Can't use resolve() since we don't want to write to the
        *-- working .SCX
        PRIVATE jcRetVal
        jcRetVal = wordsearch(c_TabWidth, .T.)
        *[ CHANGE WDP 12:23:14 on October 3, 1994
        *[ Used gnTbWidth instead of STR( cnTABWidth, 7, 3)
        gaTABWidths[ jnI] = IIF( m.jcRetVal = CHR(0), ;
                                 gnTbWidth, ;
                                 VAL( m.jcRetVal))

        USE
     ENDSCAN

     GO m.jnOldRec
     SELECT ( m.jnOldArea)

     SCATTER MEMVAR MEMO
     m.gnWINHeight= m.gnWINHeight + ;
                    m.gnTBSCXHeader + ;
                    m.gnTBSCXFooter + ;
                    m.gnTBHeight

     m.gnWINWidth = m.gnWINWidth + 2* m.gnTBSCXMargin

     *-- =============================================
     *-- Reckon setupcode #SECTIONS...
     *-- =============================================
     m.jnAtPos1 = ATC( "#SECTION 1", setupcode)
     m.jnAtPos2 = ATC( "#SECTION 2", setupcode)

     m.jcSetup1 = IIF( jnAtPos1>0 , ;
                       IIF( jnAtPos2>0, ;
                            LEFT( setupcode, m.jnAtPos2-1), ;
                            setupcode), ;
                       "#SECTION 1" + ;
                       m.cr_lf+ ;
                       IIF( jnAtPos2>0, ;
                            LEFT( setupcode, m.jnAtPos2-1), ;
                            setupcode)  )

     m.jcSetup2 = IIF( jnAtPos2>0, SUBS( setupcode, jnAtPos2), "#SECTION 2" + m.cr_lf)

     jcCntnrProps = '   '
     jcCntnrProps = jcCntnrProps + ;
                    IIF( wordsearch( m.c_TABNoFloat, .T.)= ccNull, ;
                        "FLOAT ", ;
                        "NOFLOAT ")

     jcCntnrProps = jcCntnrProps + ;
                    IIF( wordsearch( m.c_TABNoClose, .T.)= ccNull, ;
                        "CLOSE ", ;
                        "NOCLOSE ")

     jcCntnrProps = jcCntnrProps + ;
                    IIF( wordsearch( m.c_TABMinimize,.T.)= ccNull, ;
                         "NOMINIMIZE ", ;
                         "MINIMIZE ")

     jcCntnrProps = jcCntnrProps + ;
                    IIF( wordsearch( m.c_TABZoom,    .T.)= ccNull, ;
                        "NOZOOM ", ;
                        "ZOOM ")

     jcCntnrProps = jcCntnrProps + ;
                    IIF( wordsearch( m.c_TABDouble,  .T.)= ccNull, ;
                         " ", ;
                         "DOUBLE ")

     *--    Define and activate the Container window
     m.jcSetup =  m.jcSetup1 + ;
                  m.cr_lf + ;
                  "DEFINE WINDOW " + m.gcTBWindowName + " " + CHR(59) + ;
                  m.cr_lf + ;
                  "  AT 0.000, 0.000 " + CHR(59) + ;
                  m.cr_lf + ;
                  "  SIZE " + STR(m.gnWINHeight + m.gnTBHeader + m.gnTBFooter, 7, 3)+ ;
                  ", " + ;
                  STR(m.gnWINwidth+ (2 * m.gnTBMargin), 7, 3) + CHR(59) + ;
                  m.cr_lf + ;
                  "  FONT [" + TRIM( fontface) + "], " + ALLTRIM( STR( fontsize))+ " "  + CHR(59) + ;
                  m.cr_lf + ;
                  "  STYLE [" + cfontstyle( fontstyle) + "] " + CHR(59) + ;
                  m.cr_lf + ;
                  "  TITLE "+ m.jcTitle + CHR(59) + ;
                  m.cr_lf + ;
                  jcCntnrProps + CHR(59) + ;
                  m.cr_lf + ;
                  "  COLOR RGB(,,,192,192,192) "  + ;
                  m.jcWClause + ;
                  m.cr_lf + ;
                  "MOVE WINDOW " + m.gcTBWindowName + " TO 800, 800" + ;
                  cr_lf + ;
                  "ACTIVATE WINDOW " + m.gcTBWindowName + ;
                  cr_lf + ;
                  m.jcSetup2

     *-- Now, reckon the TAB layout
     m.jnCardWidth = m.gnWINwidth+ (2 * m.gnTBMargin)

     *-- Figure out how many TABS that allows
     m.jnNumTabs = INT( m.jnCardWidth/ ( ( sumTBWidth()/ (m.screens- NumSkipped())) + m.gnTBSpacing))

     *-- Crap out, for now, if too many tabs are required for a single row
     *?  ER:  Why not increase the width...
     IF m.screens-NumSkipped() > m.jnNumTabs
        WAIT WINDOW "Error, Too many TABS for maximum SCX Width"
        RETURN .F.
     ENDIF

     *-- Screen 1 Setup
     REPLACE setupcode WITH m.jcSetup

     *-- Screen 1 Cleanup
     REPLACE PROCCODE WITH ;
          "RELEASE WINDOW " + m.gcTBWindowName + ;
          cr_lf + ;
          PROCCODE + ;
          cr_lf + ;
          "FUNCTION TABWHEN" + ;
          cr_lf+ "*-- Automatically added by TABS.PRG" + ;
          cr_lf + ;
          "PARAMETER tcScreen" + ;
          cr_lf + ;
          "IF MDOWN()" + ;
          cr_lf + ;
          [ DO TabValid WITH tcScreen ] + ;
          cr_lf + ;
          "ENDIF" + ;
          cr_lf + ;
          "RETURN .T." + ;
          cr_lf + ;
          cr_lf + ;
          "FUNCTION TabValid" + ;
          cr_lf+ "*-- Automatically added by TABS.PRG" + ;
          cr_lf + ;
          "PARAMETER tcScreen" + ;
          cr_lf + ;
          "PRIVATE m.jcScreenObj" + ;
          cr_lf + ;
          [ jcScreenObj = "m.___TABS" + REPLICATE( tcScreen, 2)] + ;
          cr_lf + ;
          "_CUROBJ = OBJNUM( &jcScreenObj) " + ;
          cr_lf + ;
          "RETURN .T." + ;
          cr_lf + ;
          cr_lf + ;
          "FUNCTION goobj" + ;
          cr_lf+ "*-- Automatically added by TABS.PRG" + ;
          cr_lf + ;
          "PARAMETER tnObj" + ;
          cr_lf + ;
          "_CUROBJ = tnObj" + ;
          cr_lf + ;
          "RETURN .T." + ;
          cr_lf

     *--<RJS> Added procedure tabnotch
     REPLACE PROCCODE WITH PROCCODE+ ;
          cr_lf+ "PROCEDURE tabnotch" + ;
          cr_lf+ "*-- Automatically added by TABS.PRG" + ;
          cr_lf+ "PARAMETERS tnUlRow, tnUlCol, tnTabWidth, tnNotchWidth, tnPenSize, tcWfontface, tnWfontsize, tcWfontstyle" + ;
          cr_lf+ "PRIVATE jnPixelHeight, jnPixelWidth, jnI, jnWorkRow" + ;
          cr_lf+ "jnPixelHeight = 1/FONTMETRIC( 1, tcWfontface, tnWfontsize, tcWfontstyle)" + ;
          cr_lf+ "jnPixelWidth  = 1/FONTMETRIC( 6, tcWfontface, tnWfontsize, tcWfontstyle)" + ;
          cr_lf+ "jnWorkRow = tnUlRow" + ;
          cr_lf+ "@ jnWorkRow, tnUlCol TO jnWorkRow, tnUlCol+ jnPixelWidth* tnNotchWidth PEN 0 COLOR RGB( 192, 192, 192,,,)"

     REPLACE PROCCODE WITH PROCCODE+ ;
          cr_lf+ "FOR jnI = 1 TO tnNotchWidth" + ;
          cr_lf+ "  jnWorkRow = jnWorkRow+ jnPixelHeight" + ;
          cr_lf+ "  @ jnWorkRow, tnUlCol TO jnWorkRow, tnUlCol+ jnPixelWidth* ( tnNotchWidth- jnI) PEN 0 COLOR RGB( 192, 192, 192,,,)" + ;
          cr_lf+ "  @ jnWorkRow, tnUlCol+ jnPixelWidth* ( tnNotchWidth- jnI) TO jnWorkRow, tnUlCol+ jnPixelWidth* ( tnNotchWidth- jnI+1) PEN 0 COLOR RGB( 0, 0, 0,,,)" + ;
          cr_lf+ "  @ jnWorkRow, tnUlCol+ jnPixelWidth* ( tnNotchWidth- jnI+ 1) TO jnWorkRow, tnUlCol+ jnPixelWidth* ( tnNotchWidth- jnI+ 1+ tnPenSize) PEN 0 COLOR RGB( 255, 255, 255,,,)" + ;
          cr_lf+ "ENDFOR"

     REPLACE PROCCODE WITH PROCCODE+ ;
          cr_lf+ "jnWorkRow = tnUlRow" + ;
          cr_lf+ "tnUlCol = tnUlCol+ tnTabWidth" + ;
          cr_lf+ "@ jnWorkRow, tnUlCol- jnPixelWidth* tnNotchWidth TO jnWorkRow, tnUlCol PEN 0 COLOR RGB( 192, 192, 192,,,)" + ;
          cr_lf+ "FOR jnI = 1 TO tnNotchWidth- 1" + ;
          cr_lf+ "  jnWorkRow = jnWorkRow+ jnPixelHeight" + ;
          cr_lf+ "  @ jnWorkRow, tnUlCol- jnPixelWidth* ( tnNotchWidth- jnI+ 1) TO jnWorkRow, tnUlCol- jnPixelWidth* ( tnNotchWidth- jnI) PEN 0 COLOR RGB( 0, 0, 0,,,)" + ;
          cr_lf+ "  @ jnWorkRow, tnUlCol- jnPixelWidth* ( tnNotchWidth- jnI) TO jnWorkRow, tnUlCol PEN 0 COLOR RGB( 192, 192, 192,,,)" + ;
          cr_lf+ "ENDFOR" + ;
          cr_lf

  ENDIF

  *-- Bailout: *:TABSKIP in setup snippet
  *-- With thanks to Dan Pollak
  IF m.ScxCount > 1 AND gaSkipped( m.ScxCount)
     GO BOTTOM
     DO tabsCleanup
     RETURN
  ENDIF
  *-- =============================================
  *--    2. Write data to every screen.  We're on
  *--       the header record.
  *-- =============================================
  *-- Commit the screen-size memvars, make screen borderless, titleless, featureless, and grey
  REPLACE height    WITH m.gnWINHeight , ;
          width     WITH m.gnWINWidth, ;
          fontface  WITH gaFont[ 1, 4], ;
          fontsize  WITH gaFont[ 1, 5], ;
          fontstyle WITH gaFont[ 1, 6], ;
          float     WITH .F., ;
          close     WITH .F., ;
          minimize  WITH .F., ;
          border    WITH 0, ;
          tag       WITH '', ;
          fillred   WITH 192, ;
          fillgreen WITH 192, ;
          fillblue  WITH 192

  *-- Put the top left corner of screens to the
  *-- the new position
  REPLACE hpos WITH m.gnTBMargin ;
          vpos WITH m.gnTBHeader

  *? Suggestion by WDP -- Make a directive for this
  *  REPLACE center WITH .F.

  REPLACE setupcode WITH "#WCLAUSES IN WINDOW " + m.gcTBWindowName + ;
          cr_lf + ;
          "*:NOSCNOBJ" + ;
          cr_lf + ;
          setupcode

  m.SCNOBJ = "OFF"

  IF scxcount=1
    IF WhenType = 0   && Expression
       REPLACE when WITH "MOVE WINDOW " + m.gcTBWindowName + " CENTER" + cr_lf + "RETURN " + WHEN, ;
               whentype WITH 1
    ELSE
       REPLACE when WITH "MOVE WINDOW " + m.gcTBWindowName + " CENTER" + cr_lf + WHEN, ;
               whentype WITH 1
    ENDIF
  ENDIF

  *-- Adjust the initial hpos, vpos, height, width of the non-header records in the screen
  *-- with values for the new, normalized font
  SCAN FOR objtype <> 1 AND NOT DELETED()
     REPLACE hpos WITH hpos * gaFont[ m.scxcount, 3], ;
             vpos WITH vpos * gaFont[ m.scxcount, 2]

     DO CASE
     CASE INLIST( objtype, cnPicture, cnInvButton, cnLine, cnBox)
        REPLACE height    WITH height * gaFont[ m.scxcount, 2], ;
                width     WITH width  * gaFont[ m.scxcount, 3], ;
                fontface  WITH gaFont[ 1,4], ;
                fontsize  WITH gaFont[ 1,5], ;
                fontstyle WITH gaFont[ 1,6]

     CASE INLIST( objtype, cnCheckBox) AND "B" $ SUBS( Picture, 1, 4)
        REPLACE height    WITH height * gaFont[ m.scxcount, 2], ;
                width     WITH width  * gaFont[ m.scxcount, 3], ;
                fontface  WITH gaFont[ 1,4], ;
                fontsize  WITH gaFont[ 1,5], ;
                fontstyle WITH gaFont[ 1,6]

     CASE INLIST( objtype, cnPushButton, cnRadioButton)
        *-- could be a picture
        IF "B" $ SUBS( Picture, 1, 4)
           REPLACE height     WITH height * gaFont[ m.scxcount, 2], ;
                   width      WITH width  * gaFont[ m.scxcount, 3], ;
                   fontface   WITH gaFont[ 1,4], ;
                   fontsize   WITH gaFont[ 1,5], ;
                   fontstyle  WITH gaFont[ 1,6]
           *-- Horizontal or vertical
           IF "V" $ SUBS( Picture, 1, 5)
              REPLACE spacing WITH spacing * gaFont[ m.scxcount, 2]
           ELSE
              REPLACE spacing WITH spacing * gaFont[ m.scxcount, 3]
           ENDIF

        ENDIF

     ENDCASE

  ENDSCAN

  *-- Now ajust position of the other screen records
  *-- in the new form
  SCAN FOR objtype <> 1 AND NOT DELETED()
     REPLACE hpos WITH hpos + m.gnTBSCXMargin, ;
             vpos WITH vpos + m.gnTBSCXHeader + m.gnTBHeight
  ENDSCAN

  LOCATE FOR objtype= 1
  *-- =============================================
  *-- *:TABREPLICATE CODE
  *-- [smb] September 18, 94
  *-- =============================================
  IF m.scxcount = 1
     COPY TO _REPLICA FOR "*:TABREPLICATE" $ UPPER( comment)
  ELSE
     IF FILE( "_REPLICA.DBF") AND ( ! "*:TABREPLICATE OFF" $ UPPER( setupcode))
        APPEND FROM _REPLICA
     ENDIF
  ENDIF

  LOCATE FOR objtype= 1
  *-- =============================================
  *-- Now, add the card border and invisible buttons
  *-- =============================================
  *-- First, remember the new .SCX metrics
  SCATTER MEMVAR MEMO
  m.jnHFudge = m.gnTBPen/FONTMETRIC( 6, TRIM( m.fontface), m.fontsize, cfontstyle( m.fontstyle))
  m.jnVFudge = m.gnTBPen/( FONTMETRIC( 1, TRIM( m.fontface), m.fontsize, cfontstyle( m.fontstyle)) )

  *-- Insert Invisible Buttons
  m.jcScnNo = ALLTRIM(STR( scxcount,2))
  m.jnIBVShrink = 0.20

  FOR jnI = 1 to m.Screens
     IF gaSkipped[ jnI]
       LOOP
     ENDIF
     =_insrec()
     REPLACE platform  WITH platform_, ;
             objtype   WITH cnInvButton, ;
             objcode   WITH 1, ;
             hpos      WITH 1 + ((jni- numskipped( jni) -1) * m.gnTBSpacing) + sumTBWidths( jnI-1), ;
             vpos      WITH m.gnTBHeight * m.jnIBVShrink, ;
             name      WITH "m.___Tabs" + CHR( 64+ m.ScxCount- numskipped(  m.ScxCount))+ CHR( 64+jnI- numskipped( jnI)), ;
             height    WITH m.gnTBHeight*( 1-jnIBVShrink), ;
             width     WITH gaTABWidths[ jnI] -2, ;
             spacing   WITH m.gnTBSpacing+1, ;
             picture   WITH ["@*IHN "], ;
             fontface  WITH m.fontface, ;
             fontsize  WITH m.fontsize, ;
             fontstyle WITH m.fontsTYLE, ;
             when      WITH "TABWHEN( [" + CHR(64+ jnI- numskipped( jnI))+ "])", ;
             whentype  WITH 0, ;
             valid     WITH "TABVALID( [" + CHR(64+ jnI- numskipped( jnI))+ "])", ;
             validtype WITH 0

   ENDFOR

  *--<RJS> Begin Modification
  *--<RJS> Adjust invisible button size to accomodate small fonts
  IF .F.
     REPLACE hpos    WITH ( m.gnTBPen+1)/FONTMETRIC( 6,TRIM( m.fontface),m.fontsize,cfontstyle( m.fontstyle)), ;
             vpos    WITH ( m.gnTBPen+1)/FONTMETRIC( 1,TRIM( m.fontface),m.fontsize,cfontstyle( m.fontstyle)), ;
             spacing WITH m.gnTBSpacing + hpos * 2, ;
             width   WITH m.gnTBWidth - hpos * 2, ;
             height  WITH m.gnTBHeight - vpos
  ENDIF
  *--<RJS> End Modification

  *-- Draw curved boxes around invisible buttons
  FOR m.jni = 1 TO m.screens
     IF gaSkipped[ jnI]
       LOOP
     ENDIF

     =_insrec()

     *--<RJS> Begin Modification
     *--<RJS> use *:3D n BOX instead of *:3D RAISED for the tabs
     *[][]
     REPLACE platform  WITH platform_, ;
             objtype   WITH cnBOX, ;
             objcode   WITH 4, ;
             vpos      WITH 0 , ;
             hpos      WITH ( jni- numskipped( jni)- 1) * m.gnTBSpacing + sumTBWidth( jnI-1), ;
             width     WITH gaTABWidths[ jnI], ;
             height    WITH 2 * m.gnTBHeight + 2* m.jnVFudge, ;
             pensize   WITH 0, ;
             penpat    WITH 8, ;
             penred    WITH -1, ;
             pengreen  WITH -1, ;
             penblue   WITH -1, ;
             fillred   WITH -1, ;
             fillgreen WITH -1, ;
             fillblue  WITH -1, ;
             comment   WITH "*:3D " + str( iif( m.jni= m.scxcount, m.gnTBPen, max( 1, m.gnTBPen- 1))) + " BOX"

     *--<RJS> Begin modification
     *--Insert a thin line around each tab
     =_insrec()
     *-- Thanks to Dan Pollak for the suggestion to use
     *-- GENSCRNX.InsCmd() to give us FP2.5 compatibility
     REPLACE platform  WITH platform_, ;
             objtype   WITH cnBOX, ;
             objcode   WITH 4, ;
             vpos      WITH 0 , ;
             hpos      WITH ( jnI- numskipped( jni)- 1) * gnTBSpacing + sumTBWidth( jnI-1), ;
             width     WITH gaTABWidths[ jnI], ;
             height    WITH 2 * ( gnTBHeight + jnVFudge), ;
             pensize   WITH 1, ;
             penpat    WITH 8, ;
             penred    WITH 0, ;
             pengreen  WITH 0, ;
             penblue   WITH 0, ;
             fillred   WITH -1, ;
             fillgreen WITH -1, ;
             fillblue  WITH -1, ;
             comment   WITH "*:no3d"

     =InsCmd("do tabnotch WITH " + ;
              ALLTRIM( str( vpos, 10, 3)) + ", " + ;
              ALLTRIM( str( hpos, 10, 3)) + ", " + ;
              ALLTRIM( str( width, 10, 3)) + ", 4," + ;
              str( IIF( m.jni= m.scxcount, m.gnTBPen, MAX( 1, m.gnTBPen- 2))) + ", " + ;
              "[" + m.fontface + "], " + ALLTRIM( str( m.fontsize)) + ", [" + ;
              cfontstyle(m.fontstyle) + "]",1)

       *--<RJS> End modification
  ENDFOR

  *-- Insert a form box
  =_insrec()
  REPLACE platform  WITH platform_, ;
          objtype   WITH cnBOX, ;
          objcode   WITH 4, ;
          vpos      WITH m.gnTBHeight, ;
          hpos      WITH 0, ;
          width     WITH m.Width-jnHFudge, ;
          height    WITH m.Height-gnTBHeight-jnVFudge, ;
          pensize   WITH m.gnTBPen, ;
          penpat    WITH 8, ;
          penred    WITH -1, ;
          pengreen  WITH -1, ;
          penblue   WITH -1, ;
          fillred   WITH -1, ;
          fillgreen WITH -1, ;
          fillblue  WITH -1, ;
          comment   WITH "*:3D " + ALLTRIM( STR( m.gnTBPen )) +;
                         ",'','','','','','','','','',0,0,0" + ;
                         " BOX"

  m.jnBoxWidth = width
  m.jnBoxVPos  = vpos
  m.jnBoxHeight= height

  *-- Lay icon and text inside the tabs
  FOR m.jni = 1 TO m.screens
     IF gaSkipped[ jnI]
       LOOP
     ENDIF

     *-- Icon
     IF NOT EMPTY( gaTABIcon(m.jni))
        =_insrec()
        REPLACE picture   WITH "["+ gaTABIcon( m.jnI)+ "]", ;
                vpos      WITH gnTBHeight/2 - 0.4, ;
                hpos      WITH ( jni- numskipped( jni)-1) * gnTBSpacing + sumTBWidth[ jnI-1]+ 1, ;
                width     WITH MIN( gaTABWidths[ jnI]-2, TXTWIDTH( gaTABLabel( m.jnI), m.FontFace, m.FontSize, cfontstyle( IIF( m.jni=m.scxcount, 1,0)) )), ;
                height    WITH m.gnTbHeight, ;
                fontface  WITH m.FontFace, ;
                fontsize  WITH m.FontSize, ;
                fontstyle WITH IIF( m.jni=m.scxcount, 1,0) ;
                comment   WITH "*:NO3D", ;
                objtype   WITH 17
     ENDIF

     *-- Text
     IF NOT EMPTY( gaTABLabel( m.jni))
        =_insrec()
        REPLACE expr      WITH ALLTRIM( gaTABLabel( m.jnI)), ;
                vpos      WITH m.gnTBHeight/2 - 0.4, ;
                hpos      WITH ( jni- numskipped( jni)-1) * gnTBSpacing + sumTBWidth[ jnI-1]+ 1, ;
                width     WITH MIN( gaTABWidths[ jnI]-2, TXTWIDTH( gaTABLabel( m.jnI), m.FontFace, m.FontSize, cfontstyle( IIF(  m.jni=m.scxcount, 1,0)) )), ;
                height    WITH 1, ;
                fontface  WITH m.FontFace, ;
                fontsize  WITH m.FontSize, ;
                fontstyle WITH IIF( m.jni=m.scxcount, 1,0) ;
                comment   WITH "*:NO3D"

         IF gcTBTextPos <> "LEFT"
            *--<RJS> Begin Modification
            *--<RJS> Place tab text centered within the tab
            REPLACE expr  WITH ALLTRIM( gaTABLabel[ m.jnI]) ,;
                    width WITH TXTWIDTH( ALLTRIM( gaTABLabel[ m.jnI]), fontface, fontsize, cfontstyle( fontstyle))
            m.jnWidth = width

            IF fontstyle=1
              m.jnWidth = m.jnWidth * FONTMETRIC( 6, fontface, fontsize, "B") / FONTMETRIC( 6, fontface, fontsize)
            ENDIF

            REPLACE hpos WITH ( jnI-1) * m.gnTBSpacing + sumTBWidth( jnI-1)+ jnHFudge + 1+ ( gaTABWidths[ jnI] - m.jnWidth) / 2

            *--<RJS> End Modification

         ENDIF
     ENDIF
  ENDFOR

  *-- =============================================
  *-- Cosmetic fix to the top line to break the
  *-- line so tab & form are contiguous
  *-- =============================================
  =_insrec()
  REPLACE platform  WITH platform_, ;
          objtype   WITH cnLine, ;
          pensize   WITH m.gnTBPen, ;
          vpos      WITH m.jnBOXVPos, ;
          hpos      WITH ( m.scxcount- numskipped( m.ScxCount) -1) * m.gnTBSpacing + sumTBWidth( m.scxcount- 1) + m.jnHFudge, ;
          width     WITH gaTABWidths[ m.scxcount]- 2* jnHFudge, ;
          style     WITH 1, ;
          penpat    WITH 8, ;
          penred    WITH 192, ;
          pengreen  WITH 192, ;
          penblue   WITH 192, ;
          fillred   WITH 192, ;
          fillgreen WITH 192, ;
          fillblue  WITH 192, ;
          comment   WITH "*:NO3D"


  *--<RJS> Begin modification
  *--<RJS> fixup the intersection with the tab and the thin border on the card
  IF m.gnTBPen > 1
     =_insrec()
     REPLACE platform  WITH platform_, ;
             objtype   WITH cnLine, ;
             pensize   WITH 1, ;
             vpos      WITH m.jnBOXVPos, ;
             width     WITH ( m.gnTBPen-1)/font( 6,m.fontface,m.fontsize,cfontstyle( m.fontstyle)), ;
             hpos      WITH ( m.scxcount- numskipped( m.ScxCount) -1)* m.gnTBSpacing+ sumTBWidth( m.scxcount- 1)+ m.jnHFudge - width, ;
             style     WITH 1, ;
             penpat    WITH 8, ;
             penred    WITH 255, ;
             pengreen  WITH 255, ;
             penblue   WITH 255, ;
             fillred   WITH -1, ;
             fillgreen WITH -1, ;
             fillblue  WITH -1, ;
             comment   WITH "*:NO3D"

     =_insrec()
     REPLACE platform  WITH platform_, ;
             objtype   WITH cnLine, ;
             pensize   WITH 1, ;
             vpos      WITH m.jnBOXVPos, ;
             width     WITH ( m.gnTBPen-1)/font( 6,m.fontface,m.fontsize,cfontstyle( m.fontstyle)), ;
             hpos      WITH ( m.scxcount-1) * m.gnTBSpacing + sumTBWidth( m.scxcount-1) + gaTABWidths[ m.scxCount] - jnHFudge, ;
             style     WITH 1, ;
             penpat    WITH 8, ;
             penred    WITH 128, ;
             pengreen  WITH 128, ;
             penblue   WITH 128, ;
             fillred   WITH -1, ;
             fillgreen WITH -1, ;
             fillblue  WITH -1, ;
             comment   WITH "*:NO3D"

  ENDIF
  *--<RJS> End modification

  *-- Do we want to avoid tabing the stack?
   IF m.gcTBStack= "OFF"
      LOCATE
      =_insrec()
  *   REPLACE platform   WITH m.platform_, ;
              objtype    WITH 20, ;
              objcode    WITH 1,;
              name       WITH "m.NoStack0"+ ALLTRIM( STR( m.scxcount)), ;
              vpos       WITH 0, ;
              hpos       WITH 0, ;
              height     WITH 0, ;
              width      WITH 0, ;
              boxchar    WITH '', ;
              fillchar   WITH '', ;
              scheme     WITH 0,;
              scheme2    WITH -1, ;
              colorpair  WITH '', ;
              expr       WITH '', ;
              picture    WITH '"@*IHN "',;
              whentype   WITH 0, ;
              when       WITH "IIF( INLIST( LASTKEY(), 5, 19, 15), ;
                                    goobj( OBJNUM( m.NoStack1" + ALLTRIM( STR( m.scxcount))+ ")-1), ;
                                    TABVALID( [" + CHR( 64+ m.scxcount)+ "]))" ,;
              initialval WITH '', ;
              initialnum WITH 0, ;
              comment    WITH ''

      REPLACE platform   WITH m.platform_, ;
              objtype    WITH 20, ;
              objcode    WITH 1,;
              name       WITH "m.NoStack0"+ ALLTRIM( STR( m.scxcount)), ;
              vpos       WITH 0, ;
              hpos       WITH 0, ;
              height     WITH 0, ;
              width      WITH 0, ;
              boxchar    WITH '', ;
              fillchar   WITH '', ;
              scheme     WITH 0,;
              scheme2    WITH -1, ;
              colorpair  WITH '', ;
              expr       WITH '', ;
              picture    WITH '"@*IHN "',;
              whentype   WITH 0, ;
              when       WITH "IIF( INLIST( LASTKEY(), 5, 19, 15), ;
                                    goobj( OBJNUM( m.NoStack1" + ALLTRIM( STR( m.scxcount))+ ")-1), ;
                                    .F.)" ,;
              initialval WITH '', ;
              initialnum WITH 0, ;
              comment    WITH ''

      GO BOTTOM

      =insrec()
      REPLACE objtype    WITH 20, OBJCODE WITH 1,;
              name       WITH "m.NoStack1" + ALLTRIM( STR(m.scxcount)), ;
              expr       WITH '',;
              picture    WITH '"@*IHN "',;
              whentype   WITH 0, ;
              when       WITH "goobj( OBJNUM( m.NoStack0" + ALLTRIM( STR( m.scxcount))+ ")+ ;
                               IIF(LASTKEY()= 18 , ;
                                   " + ALLTRIM(str( m.scxcount))+ ", ;
                                   1))", ;
              initialval WITH '', ;
              initialnum WITH 0, ;
              comment    WITH ''
  ENDIF
  GO BOTTOM
  DO TabsCleanup
ENDIF

RETURN .T.


*!*********************************************
*!
*!       Procedure: tabsCleanup
*!
*!*********************************************
PROCEDURE tabsCleanup
*  Author............: Steven M. Black
*} Project...........: TABS
*  Created...........: 05/27/94
*  Copyright.........: (c) Steven Black Consulting, 1994
*) Description.......: PROCEDURE tabsCleanup
*)
*] Dependencies......:
*  Calling Samples...:
*  Parameter List....:
*  Returns...........:
*  Major change list.:

IF m.scxcount = m.SCREENS
   RELEASE m.gnWINHeight,   m.gnWINWidth,    m.gnTBMargin,    m.gnTBHeader,     m.gnTBFooter, ;
           m.gnTBSCXMargin, m.gnTBSCXHeader, m.gnTBSCXFooter, m.gnTBWidth,      m.gnTBSpacing, ;
           m.gnTBHeight,    m.gnTBPen,       m.gcTBStack,     m.gcTBWindowName, m.gcTBTextPos

   RELEASE gaTABLabel, gaFont, gaTabIcon, gaTabWidth, gaSkipped
   IF FILE( "_replica.dbf")
     ERASE _replica.dbf
     ERASE _replica.fpt
   ENDIF
ENDIF
RETURN

*!*********************************************
*!
*!       Procedure: Resolve
*!
*!*********************************************
PROCEDURE Resolve
*  Author............: Steven M. Black
*} Project...........: TABS
*  Created...........: 05/27/94
*  Copyright.........: (c) Steven Black Consulting, 1994
*) Description.......: PROCEDURE Resolve
*)                     Return directive settings...
*] Dependencies......:
*  Calling Samples...:
*  Parameter List....:
*  Returns...........:
*  Major change list.:

PARAMETER tcCookie, tlString
PRIVATE jcRetVal

jcRetVal= wordsearch(c_Tab&tcCookie, .T.)

IF m.jcRetVal <> CHR(0)
    REPLACE setupcode WITH strtranc(setupcode, m.c_Tab&tcCookie, m.m_Tab&tcCookie)
ELSE
    m.jcRetVal= IIF( !tlString, STR( cnTAB&tcCookie, 7, 3), UPPER( ccTAB&tcCookie))
ENDIF
RETURN IIF( !tlString, VAL( m.jcRetVal), m.jcRetVal)


*!*********************************************
*!
*!       Procedure: cfontstyle
*!
*!*********************************************
PROCEDURE cfontstyle
*  Author............: Steven M. Black
*  Version...........: 1.1 Dec 27 1994
*} Project...........: INTL
*  Created...........: 06/05/93
*  Copyright.........: (c) Steven Black Consulting, 1993
*) Description.......: PROCEDURE cfontstyle
*)                     Convert from numeric font style
*)                     to characters that are
*)                     required by FONTMETRIC()
*] Dependencies......:
*  Calling Samples...:
*  Parameter List....:
*  Returns...........:
*  Major change list.:

PARAMETER tnNumericStyle
PRIVATE m.jcRetVal

DO CASE
 CASE m.tnNumericStyle= 0
    m.jcRetVal= "N"
 CASE m.tnNumericStyle= 1
    m.jcRetVal= "B"
 CASE m.tnNumericStyle= 2
    m.jcRetVal= "I"
 OTHERWISE
    m.jcRetVal= "BI"
ENDCASE

RETURN m.jcRetVal


*!*********************************************
*!
*!       Procedure: labeltxt
*!
*!*********************************************
PROCEDURE labeltxt
*  Author............: Steven M. Black
*} Project...........: TABS
*  Created...........: 05/29/94
*  Copyright.........: (c) Steven Black Consulting, 1994
*) Description.......: PROCEDURE labeltxt
*)                     Determine the label text
*] Dependencies......:
*  Calling Samples...:
*  Parameter List....:
*  Returns...........:
*  Major change list.:


PARAMETER tcDirective, tcDefault
PRIVATE jcRetVal, jcTrial

IF EMPTY( tcDefault)
   tcDefault = ''
ENDIF

*-- Something to default to...
m.jcRetVal = tcDefault

m.jcTrial = WORDSEARCH( tcDirective, .T.)

DO CASE
*-- Specific directive
CASE m.jcTrial <> CHR(0)
   m.jcRetVal = m.jcTrial

*-- Screen Title
CASE NOT EMPTY( TAG)
   m.jcRetVal = ALLTRIM( SUBS(TAG,2, LEN(TAG)-2))

ENDCASE

IF DrvAudit( "INTL") > 0
  jcRetVal = [I( "] + jcRetVal + [")]

ELSE
  jcRetVal = "[" + jcRetVal + "]"

ENDIF

RETURN m.jcRetVal


*!*********************************************
*!
*!       Procedure: Icontxt
*!
*!*********************************************
PROCEDURE Icontxt
*  Author............: Steven M. Black
*} Project...........: TABS
*  Created...........: 05/29/94
*  Copyright.........: (c) Steven Black Consulting, 1994
*) Description.......: PROCEDURE labeltxt
*)                     Determine the label text
*] Dependencies......:
*  Calling Samples...:
*  Parameter List....:
*  Returns...........:
*  Major change list.:

PARAMETER tcDirective, tcDefault
PRIVATE jcRetVal, jcTrial

IF EMPTY( tcDefault)
   tcDefault = ''
ENDIF

*-- Something to default to...
m.jcRetVal = tcDefault

m.jcTrial = WORDSEARCH( tcDirective, .T.)

*-- Specific directive
DO CASE
CASE m.jcTrial <> CHR(0)
   m.jcRetVal = m.jcTrial

*-- Screen icon
CASE NOT EMPTY( order)
   m.jcRetVal = ALLTRIM( SUBS( order, 2, LEN(order)-2))

ENDCASE

RETURN m.jcRetVal

*!*********************************************
*!
*!       Procedure: drvaudit
*!
*!*********************************************
PROCEDURE drvaudit
*  Author............: Steven M. Black
*} Project...........:
*  Created...........: 06/01/94
*  Copyright.........: (c) Steven Black Consulting, 1994
*) Description.......: PROCEDURE drvaudit
*)                     Check for the presence of a given Driver
*] Dependencies......:
*  Calling Samples...:
*  Parameter List....:
*  Returns...........:
*                     -1 if driver not invoked at all
*                      0 if driver invoked *before* TABS
*                      1 if driver invoked after TABS
*  Major change list.:

*-- SCAN the A_SCXDRV array and report
*-- where the driver is invoked
PARAMETER tcDriver
tcdriver= UPPER( trimext( tcDriver))

PRIVATE jlDrvInvoked, jlTabsFirst, jni

m.jlTabsFirst = .F.
m.jlDrvInvoked = .F.

FOR m.jni = 1 TO ALEN( A_SCXDRV)

   IF ! m.jlTabsFirst AND ! m.jlDrvInvoked AND TYPE( "A_SCXDRV( m.jnI)") = "C" AND "TABS." $ UPPER( A_SCXDRV( m.jnI) )
      m.jlTabsFirst = .T.
      LOOP
   ENDIF

   IF TYPE("A_SCXDRV( m.jnI)") = "C" AND UPPER( tcDriver)+ "." $ UPPER( A_SCXDRV( m.jnI) )
      m.jlDrvInvoked = .T.
      EXIT
   ENDIF
ENDFOR

DO CASE
CASE m.jlDrvInvoked AND jlTabsFirst
   RETURN 1
CASE ! m.jlDrvInvoked
   RETURN -1
CASE ! m.jlTabsFirst
   RETURN 0
ENDCASE


*!*********************************************
*!
*!       Procedure: _insrec
*!
*!*********************************************
PROCEDURE _insrec
*  Author............: Steven M. Black
*} Project...........:
*  Created...........: 09/17/94
*  Copyright.........: (c) Steven Black Consulting, 1994
*) Description.......: PROCEDURE _insrec
*)                     Workaround for a sequencing problem in
*)                     GENSCRNX version 2's insrec()
*] Dependencies......:
*  Calling Samples...:
*  Parameter List....:
*  Returns...........:
*  Major change list.:

IF RECNO() < RECCOUNT()
  *-- we don't use SKIP since it respects filters...
  GOTO RECNO()+ 1
ENDIF
=insrec(-1)

RETURN


*!*********************************************
*!
*!       Procedure: NumSkipped
*!
*!*********************************************
PROCEDURE NumSkipped
*  Author............: Steven M. Black
*} Project...........:
*  Created...........: 10/8/1994
*  Copyright.........: (c) Steven Black Consulting, 1994
*) Description.......: PROCEDURE NumSkipped
*)                     Return .T. count from gaSkipped
*] Dependencies......:
*  Calling Samples...:
*  Parameter List....:
*  Returns...........:
*  Major change list.:

PARAMETER tnMaxNum

PRIVATE jnI, jnRetVal, jnRefNum
jnRetVal = 0

jnRefNum= IIF( TYPE( "tnMaxNum")= "L", ALEN( gaSkipped), tnMaxNum )

FOR jnI = 1 TO jnRefNum
  jnRetVal = jnRetVal + IIF( gaSkipped( jnI), 1, 0)
ENDFOR

RETURN jnRetVal


*!*********************************************
*!
*!       Procedure: sumTBWidth
*!
*!*********************************************
PROCEDURE sumTBWidth
*  Author............:
*} Project...........:
*  Created...........: 09/18/94
*  Copyright.........: (c) Steven Black Consulting, 1994
*) Description.......: PROCEDURE sumTBWidth
*)
*] Dependencies......:
*  Calling Samples...:
*  Parameter List....:
*  Returns...........:
*  Major change list.:

PARAMETER tnTABNo
PRIVATE jnLimit, jnRetVal, jnCounter

IF TYPE( "tnTABNo") = "L"
  tnTABNo= -1
ENDIF

jnRetVal= 0
jnLimit = tnTABNo

IF tnTABNo = -1
   jnLimit = ALEN( gaTABWidths)
ENDIF

FOR jnCounter = 1 TO jnLimit
  jnRetVal = jnRetVal + IIF( gaSkipped[ jnCounter], 0, gaTABWidths[ jnCounter])
ENDFOR

RETURN jnRetVal

* Whew!
